@@ -90,6 +90,9 @@ typedef struct rr {
                         of the same record for the same data type */
     struct rr *ip; /* If this is a PTR, MX, or NS record, pointer to the
                       ip the rr domain name points to (if applicable) */
+    js_string *ptr; /* If this is a PTR request pointing to a CNAME,
+                       we want to give them the PTR record along
+		       with the CNAME record. */
     uint16 rr_type; /* Type of resource record this entry is */
     js_string *query; /* Pointer to the query one asks to get this answer */
     js_string *data; /* The actual raw binary data for this RR */
@@ -201,3 +204,11 @@ typedef struct {
 #define NOT_IMPLEMENTED 4
 #define REFUSED 5
 
+/* The longest RRs which we allow to exist in csv1 zone files.
+   Warning: If you make this longer, then there is a significant
+   risk that the record will be longer than the 512 bytes allowed 
+   in a DNS packet.  MaraDNS does not current support DNS over TCP.
+*/
+
+#define MAX_RECORD_LENGTH 425
+
