@@ -3,13 +3,14 @@
 #include <stdio.h>
 #include "../MaraDns.h"
 #include "Compress_rrs.h" /* The description of RRs to decompress */
+#include "Compress_rrdescs.h" /* The description of RRs to decompress */
 
 /* Data structure to store the RRs in an easy-to-use format */
-typedef struct rrdesc {
+/*typedef struct rrdesc {
     int rr_num;
     char *description;
     struct rrdesc *next;
-    } rrdesc;
+    } rrdesc; */
 
 /* Some definitions */
 #define MAX_DLABEL_LEN 256
@@ -1044,3 +1045,11 @@ int decompress_data(js_string *compressed, js_string *uncompressed) {
     return JS_ERROR;
     }
 
+/* Since the compression code needs to use the same rr_formats database,
+   this bit of code exports the rr_formats hash so that the compression
+   code can use it */
+
+rrdesc **decomp_export_rrformats() {
+    return rr_formats;
+    }
+
