@@ -1030,3 +1030,41 @@ int js_readuint16(js_string *js, unsigned int offset) {
 
     }
 
+/* js_substr_append: Append a substring of one js_string object to
+   the end of another js_string object.
+
+   Input: The js_string object with the desired substring, the
+   string to append to, the offset in the source string, the
+   length (in bytes) to append
+
+   Output: JS_ERROR, JS_SUCCESS on success
+
+ */
+
+int js_substr_append(js_string *source, js_string *dest, int offset,
+                     int length) {
+
+    js_string *temp;
+    if((temp = js_create(length + 2,1)) == 0) {
+        return JS_ERROR;
+        }
+
+    if(source->unit_count < offset + length) {
+       js_destroy(temp);
+       return JS_ERROR;
+       }
+
+    if(js_substr(source,temp,offset,length) != JS_SUCCESS) {
+       js_destroy(temp);
+       return JS_ERROR;
+       }
+
+    if(js_append(temp,dest) == JS_ERROR) {
+        js_destroy(temp);
+        return JS_ERROR;
+        }
+
+    js_destroy(temp);
+    return JS_SUCCESS;
+    }
+
