@@ -20,17 +20,16 @@
 #include "JsStr.h"
 
 /* Structures that keep track of allocated memory */
-#ifdef DEBUG_MEMORY
-leak_tracker *lt_head = NULL;
-#endif /* DEBUG_MEMORY */
 #ifdef DEBUG
+#define HASH_SIZE 100000
 typedef struct lt_hash_spot {
     void *allocated_memory;
     struct lt_hash_spot *next;
+    char *de_donde; /* From where */
     int allocated_size;
     } lt_hash_spot;
 int lt_hash_initialized = 0;
-lt_hash_spot lt_hash[65536];
+lt_hash_spot lt_hash[HASH_SIZE];
 int total_allocated_memory = 0;
 #define THREADS 
 #endif /* DEBUG */
@@ -56,20 +55,21 @@ void *js_alloc(int unit_count, int unit_size) {
        bytes allocated for each piece of allocated memory */
 
     (int)hash_index = (void *)data;
-    hash_index &= 0xffff;
+    hash_index %= HASH_SIZE;
 #ifdef THREADS
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
     /* Initialize the lt hash the first time we run this routine */
     if(lt_hash_initialized == 0) {
-        for(counter = 0; counter < 65536; counter++) 
-	    lt_hash[counter].allocated_size =0;
+        for(counter = 0; counter < HASH_SIZE; counter++) 
+	    lt_hash[counter].allocated_size = 0;
         lt_hash_initialized = 1;
         }
     /* If the hash pointer is empty, use that */
     if(lt_hash[hash_index].allocated_size == 0) {
         lt_hash[hash_index].allocated_size = unit_count * unit_size;
 	lt_hash[hash_index].allocated_memory = data;
+	lt_hash[hash_index].de_donde = 0;
         lt_hash[hash_index].next = 0;
         }
     /* Otherwise, go down the linked list, place the new node at the
@@ -83,6 +83,7 @@ void *js_alloc(int unit_count, int unit_size) {
         new->allocated_size = unit_count * unit_size;
         new->next = 0;
 	new->allocated_memory = data;
+	new->de_donde = 0;
         point = lt_hash[hash_index].next;
 	if(point == 0) {
 	    lt_hash[hash_index].next = new;
@@ -109,65 +110,87 @@ void *js_alloc(int unit_count, int unit_size) {
     return data;
     }
 
-#ifdef DEBUG_MEMORY
-/* js_alloc_DEBUG: Allocate memory from the underlying OS, keeping track
-                   of where the memory is being allocated from.
-   input: The unit count and unit size of memory to allocate, a char string
-          of why we are allocating the memory
-   output: An anonymous pointer to the allocated memory
-*/
+/* js_alloc_DEBUG: Allocate memory from the underlying OS.
+   input: The unit count and unit size of memory to allocate,
+          from where they allocated this memory
+   output: An anonymous pointer to the allocated memory */
+
 void *js_alloc_DEBUG(int unit_count, int unit_size, char *whence) {
     void *data;
-    leak_tracker *node, *point;
+#ifdef DEBUG 
+    unsigned int hash_index,counter;
+    lt_hash_spot *new,*point;
     char *why;
+#endif /* DEBUG */
     data = (void *)malloc(unit_count * unit_size);
-    if(data == NULL) {
-        printf("Can not allocate memory");
-        exit(64);
-        return (void *)0;
-        }
-    /* Now, add an element to the leak tracker linked list */
-    node = (leak_tracker *)malloc(sizeof(leak_tracker));
-    if(node == NULL) {
-        printf("Can not allocate lt memory");
-        exit(64);
-        free(data);
-        return (void *)0;
-        }
-    why = (char *)malloc(strlen(whence) + 3);
-    if(why == NULL) {
-        printf("Can not allocate ltw memory");
-        exit(64);
-        free(data);
-        free(node);
-        return (void *)0;
-        }
-    strncpy(why,whence,strlen(whence) + 1);
-    node->where = data;
-    node->why = why;
-    node->next = NULL;
+#ifdef DEBUG
+    /* If debug is set, we have a 64k element hash table which uses the low 
+       16 bits of the memory allocated as the hash index.  Each element in 
+       the hash table is a linked list, which has the address and number of
+       bytes allocated for each piece of allocated memory */
+
+    (int)hash_index = (void *)data;
+    hash_index %= HASH_SIZE;
+    /* Set up the string used to tell people why this memory was
+       allocated */
+    if((why = calloc(js_strnlen(whence,2048) + 2)) == 0) {
+        printf("Aieeee, mem alloc problem\n");
+	exit(64);
+	}
+    strncpy(why,whence,js_strnlen(whence,2048) + 1);
 #ifdef THREADS
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
-    /* Put the new element at the end of the linked list */
-    if(lt_head == NULL)
-        lt_head = node;
-    else {
-        point = lt_head;
-        while(point->next != NULL)
-            point = point->next;
-        point->next = node;
+    /* Initialize the lt hash the first time we run this routine */
+    if(lt_hash_initialized == 0) {
+        for(counter = 0; counter < HASH_SIZE; counter++) 
+	    lt_hash[counter].allocated_size =0;
+        lt_hash_initialized = 1;
         }
+    /* If the hash pointer is empty, use that */
+    if(lt_hash[hash_index].allocated_size == 0) {
+        lt_hash[hash_index].allocated_size = unit_count * unit_size;
+	lt_hash[hash_index].allocated_memory = data;
+	lt_hash[hash_index].de_donde = why;
+        lt_hash[hash_index].next = 0;
+        }
+    /* Otherwise, go down the linked list, place the new node at the
+       end of the branch in question */
+    else {
+        new = malloc(sizeof(lt_hash_spot));
+	if(new == NULL) {
+	    printf("Aieeee, can not allocate memeory for lt_hash_spot!\n");
+	    exit(1);
+	    }
+        new->allocated_size = unit_count * unit_size;
+        new->next = 0;
+	new->allocated_memory = data;
+	new->de_donde = why;
+        point = lt_hash[hash_index].next;
+	if(point == 0) {
+	    lt_hash[hash_index].next = new;
+	    }
+	else {
+	    while(point->next != 0) 
+	        point = point->next;
+            point->next = new;
+	    }
+        }
+    total_allocated_memory += unit_count * unit_size;
 #ifdef THREADS
     pthread_mutex_unlock(&alloc_lock);
 #endif /* THREADS */
-    /* Finally, return the pointer to the memory the calling function
-       wanted */
+#endif /* DEBUG */
+    if(data == NULL) {
+        /* Securty: In a situtation where we can not allocate memory,
+           the subsequent behavior of the program is undefined.  Hence,
+           the best thing to do is exit then and there */
+        printf("Aieeeeee, can not allocate memory!");
+        exit(64);
+        return (void *)0;
+        }
     return data;
     }
-#else  /* DEBUG_MEMORY */
-#define js_alloc_DEBUG(x,y,z) js_alloc(x,y)
-#endif /* DEBUG_MEMORY */
 
 /* js_dealloc: Deallocate memory from the underlying OS
    input: A pointer to the memory we wish to free
@@ -178,9 +201,6 @@ int js_dealloc(void *pointer) {
     lt_hash_spot *lpoint,*llast;
     unsigned int hash_index;
 #endif /* DEBUG */
-#ifdef DEBUG_MEMORY
-    leak_tracker *npoint, *point, *last;
-#endif /* DEBUG_MEMORY */
 #ifdef DEBUG
     /* We need to find out the number of bytes that we have allocated 
        for this given address; subtract that from the total amount of
@@ -190,18 +210,24 @@ int js_dealloc(void *pointer) {
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
     (int)hash_index = (void *)pointer;
-    hash_index &= 0xffff;
+    hash_index %= HASH_SIZE;
     if(lt_hash[hash_index].allocated_memory == pointer) {
         lpoint = lt_hash[hash_index].next;
         total_allocated_memory -= lt_hash[hash_index].allocated_size;
 	if(lpoint == 0) {
 	    lt_hash[hash_index].allocated_size = 0;
 	    lt_hash[hash_index].allocated_memory = 0;
+	    if(lt_hash[hash_index].de_donde != 0)
+	        free(lt_hash[hash_index].de_donde);
+            lt_hash[hash_index].de_donde = 0;
 	    }
         else {
 	    lt_hash[hash_index].allocated_size = lpoint->allocated_size;
 	    lt_hash[hash_index].allocated_memory = lpoint->allocated_memory;
 	    lt_hash[hash_index].next = lpoint->next;
+	    if(lt_hash[hash_index].de_donde != 0)
+	        free(lt_hash[hash_index].de_donde);
+            lt_hash[hash_index].de_donde = lpoint->de_donde;
 	    free(lpoint);
 	    }
         }
@@ -229,61 +255,45 @@ int js_dealloc(void *pointer) {
         else {
 	    llast->next = lpoint->next;
 	    }
+        if(lpoint->de_donde != 0)
+	    free(lpoint->de_donde);
         free(lpoint);
 	}
 #ifdef THREADS
     pthread_mutex_unlock(&alloc_lock);
 #endif /* THREADS */
 #endif /* DEBUG */
-#ifdef DEBUG_MEMORY
-    /* If we are keeping track of allocated memory, remove the element from
-       the linked list indicating that we have allocated memory */
-#ifdef THREADS
-    pthread_mutex_lock(&alloc_lock);
-#endif /* THREADS */
-    point = lt_head;
-    last = NULL;
-    while(point != NULL) {
-        npoint = point->next;
-        /* If this is the node for the allocated memory in question, remove
-           the node */
-        if(point->where == pointer) {
-            free(point->why);
-            /* We have to handle the special case of the node being
-               the head of the linked list */
-            if(last == NULL)
-                lt_head = point->next;
-            /* This is the normal case */
-            else
-                last->next = point->next;
-            free(point);
-            }
-        else
-            last = point;
-        point = npoint;
-        }
-#ifdef THREADS
-    pthread_mutex_unlock(&alloc_lock);
-#endif /* THREADS */
-#endif /* DEBUG_MEMORY */
     free(pointer);
     return 1;
     }
 
-#ifdef DEBUG_MEMORY
+#ifdef DEBUG
 /* Function which prints to the standard output all unallocated strings which
    we have a record of */
 int js_show_leaks() {
-    leak_tracker *point;
-    point = lt_head;
-    while(point != NULL) {
-        printf("%s\n",point->why);
-        point = point->next;
+    lt_hash_spot *point;
+    int counter,total = 0;
+    for(counter = 0; counter < HASH_SIZE; counter++) {
+        point = &lt_hash[counter];
+        while(point != 0 && point->allocated_memory != 0) {
+	    if(point->de_donde != 0) {
+                printf("%s: %d bytes at %p\n",point->de_donde,
+		       point->allocated_size,point->allocated_memory);
+		total += point->allocated_size;
+		}
+            point = point->next;
+            }
         }
+    printf("Allocated memory accounted for:       %d\n",total);
+    printf("Allocated memory *not* accounted for: %d\n",
+           total_allocated_memory - total);
+    printf("Total memory allocated:               %d\n",
+           total_allocated_memory);
+    return 0;
     }
-#else  /* DEBUG_MEMORY */
+#else  /* DEBUG */
 #define js_show_leaks()
-#endif /* DEBUG_MEMORY */
+#endif /* DEBUG */
 
 /* Routine which tells us how much memory we have allocated.
    Input: None
@@ -617,7 +627,10 @@ int js_buf_read(js_file *desc) {
 /* js_buf_getline: Grab a line from an opened file (uses buffering)
    input: Pointer to file descriptor of open file, pointer to js_string to
           put line in
-   output: JS_ERROR (-1) on error, JS_SUCCESS(1) on success */
+   output: JS_ERROR (-1) on error, JS_SUCCESS(1) on success,
+           -2 if the line is too long for the string
+ */
+
 int js_buf_getline(js_file *desc, js_string *js) {
     js_string *newlines, *temp;
     int next_newln;
@@ -648,7 +661,7 @@ int js_buf_getline(js_file *desc, js_string *js) {
     /* Find the next newline character in the string */
     next_newln = js_match_offset(newlines,desc->buffer,desc->number);
     /* If we are at the end of the buffer in question w/o finding a
-       newlines...*/
+       newline... */
     if(next_newln == -2 && desc->eof == 0) {
         /* ...then we put the rest of the buffer in js, but only if
            it will fit in js.  Otherwise, we blank out js and return
@@ -683,6 +696,8 @@ int js_buf_getline(js_file *desc, js_string *js) {
         if(desc->number >= JS_BUFSIZE)
             js_buf_read(desc);
         js_destroy(newlines);
+        if(overflowed == 1)
+            return -2;
         return JS_SUCCESS;
         }
     else if(next_newln == -2) {
@@ -701,6 +716,8 @@ int js_buf_getline(js_file *desc, js_string *js) {
         desc->number = desc->buffer->unit_count + 1;
 
         js_destroy(newlines);
+        if(overflowed == 1)
+            return -2;
         return JS_SUCCESS;
         }
 
@@ -715,6 +732,10 @@ int js_buf_getline(js_file *desc, js_string *js) {
         js_str2js(js,"",0,js->unit_size);
 
     js_destroy(newlines);
+
+    if(overflowed == 1)
+        return -2;
+
     return JS_SUCCESS;
 
     }
@@ -724,7 +745,7 @@ int js_buf_getline(js_file *desc, js_string *js) {
    output: JS_ERROR on failure, JS_SUCCESS on success
    note: it's here because of strlen call */
 int js_qstr2js(js_string *js, char *string) {
-    return js_str2js(js,string,strlen(string),js->unit_size);
+    return js_str2js(js,string,js_strnlen(string,1000000),js->unit_size);
     }
 
 /* js_adduint32: Add a 32-bit number to the end of a js_string obejct
@@ -788,4 +809,23 @@ u_int32_t js_readuint32(js_string *js, unsigned int offset) {
 
     }
 
+/* js_strnlen: Determine the length of a null-terminated string, up
+   to the length determined by the limit.  This is here because strnlen
+   is, alas, not a portable string library call.  This call is originally 
+   by Matthew T. Russotto */
+
+int js_strnlen(char *s, u_int32_t limit) {
+    u_int32_t len;
+    if(limit > 2147483600 || limit < 0) { /* Just under 2 ** 31 */
+        return JS_ERROR;
+        }
+    if(s == NULL) {
+        return JS_ERROR;
+        }
+    len = 0; 
+    while(len < limit && (*s++)) {
+        len++;
+        }
+    return len; 
+    }
 
