@@ -1,5 +1,5 @@
 /* Released to the public domain 2001 by Sam Trenholme */
-/* Note: This file is encoded using iso-8859-1 encoding */
+/* Note: This file is encoded using utf-8 encoding */
 /* Note also: This is the revamped linked-list based hash which was changed
    for MaraDNS 0.8.30 */
 /* MaraHash: A series of functions to make assosciative arrays using
@@ -489,7 +489,7 @@ int mhash_autogrow(mhash *hash) {
 
     /* Integer arithmetic, ugh.  If the hash is more than 50% full,
        we grow it.  Make this mhash_mask[bits - 1] to grow at 25% size and
-       mhash_mask[bits - 2] + mhash_mask[bits - 3] to grow at 37% size */
+       mhash_mask[bits - 2] + mhash_mask[bits - 3] to grow at 37½% size */
 
     if(hash->spots >= mhash_mask[bits - 1]) {
         if(mhash_resize(hash,bits + 1) == JS_ERROR)
