@@ -125,11 +125,24 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
         goto clean;
     if(js_space_chars(blankq) == JS_ERROR)
         goto clean;
-    /* use starq as tempory string to append newlines to blankq */
+
+    /* use starq as temporary string to append newlines to blankq */
     if(js_newline_chars(starq) == JS_ERROR)
         goto clean;
     if(js_append(starq,blankq) == JS_ERROR)
         goto clean;
+
+    /* While starq has a newline in it, lets make sure the line in
+       question ends with a newline character */
+    tempp = js_match(starq,line);
+    if(tempp == JS_ERROR)
+        goto clean;
+    if(tempp == -2) {
+        js_qstr2js(name,"ERROR");
+        js_qstr2js(data,"No newline found in line");
+        goto error;
+        }
+
     /* Now, give starq its proper value */
     if(js_str2js(starq,&star,1,1) == JS_ERROR)
         goto clean;
@@ -413,7 +426,7 @@ int parse_csv1_line(js_string *line, js_string *name, js_string *data,
     error:
         js_destroy(pipeq); js_destroy(hashq); js_destroy(starq);
         js_destroy(blankq); js_destroy(field);
-        return -1;
+        return -2;
     cero:
         js_destroy(pipeq); js_destroy(hashq); js_destroy(starq);
         js_destroy(blankq); js_destroy(field);
