@@ -15,7 +15,7 @@ extern int js_space_chars();
 
 /* Keywords that are non-dictionary strings in Mara's rc file */
 
-#define KEYCOUNT 29
+#define KEYCOUNT 30
 
 char *keywords[KEYCOUNT] = {
         "bind_address", /* Address to bind to (0.0.0.0 is all addresses) */
@@ -78,6 +78,8 @@ char *keywords[KEYCOUNT] = {
 	                  to the server */
 	"reject_ptr", /* Send a bogus SOA whenever a PTR request is sent
 	                 to the server */
+	"verbose_query", /* Whether to output every query the recursive DNS
+			    server makes */
         "zone_transfer_acl" /* ACL of IPs allowed to perform zone transfers */
         };
 
