@@ -1,5 +1,5 @@
 #define L_WARNING "Aviso: "
-#define L_ONLINE " en lnea "
+#define L_ONLINE " en línea "
 #define L_F "\n"
 #define L_ZONEOPEN "No se pudo abrir el archivo de zona para la zona "
 #define L_FATAL "Error fatal en el archivo de zona "
@@ -17,4 +17,6 @@ registro SOA."
 #define L_INITCSV "csv1 hash is not correctly initialized.\nMake sure to have cs
 v1 initialized with csv1 = {}"
 #define L_BADZONE "A zone file is incorrectly named.  All zone files must end with a dot, e.g.\ncsv1[\"example.com.\"] = \"filename\".\nHere is the line in the mararc with the problem: "
+#define L_LINE_NUMBER "Line number "
+#define L_TOO_LONG " is too long in file "
 
