@@ -5,6 +5,7 @@
    it receives.
 */
 
+#include "../libs/MaraHash.h"
 #include <stdio.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -17,6 +18,9 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include "../MaraDns.h"
+#include "../dns/functions_dns.h"
+#include "../parse/functions_parse.h"
+
 /* All of the labels */
 #include "askmara_labels_en.h"
 /* BEGIN RNG USING CODE */
@@ -31,6 +35,11 @@ int timeout = 21;
 /* So -Wall doesn't gag */
 extern int decomp_init();
 
+int csv1_answer(js_string *uindata, int *place, js_string *query);
+int csv1_compatible_output(js_string *uindata, js_string *query);
+int verbose_output(js_string *uindata);
+int out_answer(js_string *uindata,int *place);
+
 /* Octal escape any non-ASCII sequences in the askmara output; almost idential
    to show_esc_stdout */
 int escape_stdout(js_string *js) {
@@ -63,11 +72,10 @@ u_int16_t gen_id(char *hostname) {
     /* BEGIN RNG USING CODE */
     /* May as well bring out the bug guns (e.g. our secure RNG) */
     BYTE r_inBlock[17],r_outBlock[17],r_binKey[17];
-    BYTE r_keyMaterial[320];
     keyInstance r_keyInst;
     cipherInstance r_cipherInst;
     unsigned char crypto_key[34];
-    int desc, len, counter;
+    int counter;
     char *mc1, *mc2;
     /* END RNG USING CODE */
     int ret = 4321;
@@ -124,17 +132,16 @@ int harderror(char *msg) {
     exit(1);
     }
 
-main(int argc, char **argv) {
+int main(int argc, char **argv) {
     char *server_address = NULL;
-    struct sockaddr_in dns_udp, server;
+    struct sockaddr_in dns_udp;
     int len_inet; /* Length */
     int s; /* Socket */
-    int q; /* Used for binding, etc */
     js_string *outdata; /* Outgoing data */
     js_string *indata, *uindata; /* Incoming data (uncompressed version) */
     q_header header; /* header data */
     q_question question;
-    int counter,place,count;
+    int place,count;
     fd_set rx_set; /* Using select() because if its timeout option */
     int maxd;      /* select() */
     struct timeval tv;  /* select() */
@@ -289,6 +296,7 @@ main(int argc, char **argv) {
     else
         csv1_compatible_output(uindata,question.qname);
 
+    return 0;
     }
 
 /* Verbose form of askmara output
@@ -440,7 +448,7 @@ int out_answer(js_string *uindata,int *place) {
                    L_DOT,*(uindata->string + *place + 2),
                    L_DOT,*(uindata->string + *place + 3),L_NEWLINE,L_NEWLINE);
             }
-        else if(rr_hdr.type = RR_TXT) {
+        else if(rr_hdr.type == RR_TXT) {
             printf("%s",L_TXT); /* Text String */
             read_txt(uindata,mx.exchange,*place);
             escape_stdout(mx.exchange);
@@ -451,7 +459,8 @@ int out_answer(js_string *uindata,int *place) {
 
         *place += rr_hdr.rdlength; /* To do: read the RD data itself for
                                       all MaraDNS supported RRs */
-        }
+        return 0;
+	}
 
 /* csv1-compatible form of askmara output
    input: Pointer to js_string object with uncompressed UDP data,
@@ -570,15 +579,15 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
             harderror(L_READSOA);
         hname_translate(soa.mname,RR_SOA);
         hname_translate(soa.rname,RR_MX);
-        printf("|",L_S_MNAME); /* SOA mname */
+        printf("|"); /* SOA mname */
         escape_stdout(soa.mname);
-        printf("|",L_NEWLINE,L_S_RNAME); /* SOA rname */
+        printf("|"); /* SOA rname */
         escape_stdout(soa.rname);
-        printf("|%u",soa.serial,L_NEWLINE);
-        printf("|%d",soa.refresh,L_NEWLINE);
-        printf("|%d",soa.retry,L_NEWLINE);
-        printf("|%d",soa.expire,L_NEWLINE);
-        printf("|%u\n",soa.minimum,L_NEWLINE);
+        printf("|%u",soa.serial);
+        printf("|%d",soa.refresh);
+        printf("|%d",soa.retry);
+        printf("|%d",soa.expire);
+        printf("|%u\n",soa.minimum);
         }
     else if(rr_hdr.type == RR_MX) {
         if(uindata->unit_count < *place + 2)
@@ -586,7 +595,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
         mx.preference = ((*(uindata->string + *place) & 0xff) << 8) |
                          (*(uindata->string + *place + 1) & 0xff);
         read_ns(uindata,mx.exchange,*place + 2);
-        printf("|%d",mx.preference,L_NEWLINE); /* MX pref. */
+        printf("|%d",mx.preference); /* MX pref. */
         hname_translate(mx.exchange,-2); /* Makes the char a pipe */
         escape_stdout(mx.exchange);
         printf("\n");
@@ -607,7 +616,7 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
                L_DOT,*(uindata->string + *place + 2),
                L_DOT,*(uindata->string + *place + 3));
         }
-    else if(rr_hdr.type = RR_TXT) {
+    else if(rr_hdr.type == RR_TXT) {
         printf("|"); 
         read_txt(uindata,mx.exchange,*place);
         escape_stdout(mx.exchange);
@@ -619,5 +628,6 @@ int csv1_answer(js_string *uindata, int *place, js_string *query) {
 
     *place += rr_hdr.rdlength; /* To do: read the RD data itself for
                                   all MaraDNS supported RRs */
+    return 0;
     }
 
