@@ -2,7 +2,7 @@
 
 #include <stdio.h>
 
-main() {
+int main() {
         unsigned char a;
         while(!feof(stdin)) {
                 a = getc(stdin);
@@ -14,6 +14,7 @@ main() {
                         printf("%c",a);
 
                 }
+	return 0;
         }
 
 
