@@ -48,10 +48,10 @@ int main(int argc, char **argv) {
     unsigned char len[2];
 
     if((argc < 3 && argc > 4) || argv[1] == 0 || argv[2] == 0) {
-        harderror(L_USAGE); /* "Usage: getzone zone_name zone_server_IP" */
+        harderror(L_USAGE_GZ); /* "Usage: getzone zone_name zone_server_IP" */
         }
     if(argc == 4 && argv[3] == 0) {
-        harderror(L_USAGE); /* "Usage: getzone zone_name zone_server_IP" */
+        harderror(L_USAGE_GZ); /* "Usage: getzone zone_name zone_server_IP" */
 	}
 
     /* Set up an option to change the query class */
@@ -145,7 +145,7 @@ int main(int argc, char **argv) {
     if((soa.mname = js_create(257,1)) == 0)
         harderror(L_MNAME); /* "Could not create soa.mname" */
     if((soa.rname = js_create(257,1)) == 0)
-        harderror(L_RNAME); /* "Could not create soa.rname" */
+        harderror(L_RNAME_GZ); /* "Could not create soa.rname" */
 
     while(recv(sock,len,2,MSG_WAITALL) == 2) {
 
